this.ai_weakness <- this.inherit("scripts/ai/tactical/behavior", {
	m = {
		TargetTile = null,
		Danger = null,
		ScoreBonus = 1.0,
		PossibleSkills = [
			"actives.weakness"
		],
		Skill = null
	},
	function create()
	{
		this.m.ID = this.Const.AI.Behavior.ID.Weakness;
		this.m.Order = this.Const.AI.Behavior.Order.Weakness;
		this.m.IsThreaded = true;
		this.behavior.create();
	}

	function onEvaluate( _entity )
	{
		// Function is a generator.
		this.m.Skill = null;
		this.m.TargetTile = null;
		this.m.Danger = null;
		this.m.ScoreBonus = 0.0;
		local score = this.getProperties().BehaviorMult[this.m.ID];

		if (_entity.getActionPoints() < this.Const.Movement.AutoEndTurnBelowAP)
		{
			return this.Const.AI.Behavior.Score.Zero;
		}

		if (_entity.getMoraleState() == this.Const.MoraleState.Fleeing)
		{
			return this.Const.AI.Behavior.Score.Zero;
		}

		if (!this.getAgent().hasKnownOpponent())
		{
			return this.Const.AI.Behavior.Score.Zero;
		}

		this.m.Skill = this.selectSkill(this.m.PossibleSkills);

		if (this.m.Skill == null)
		{
			return this.Const.AI.Behavior.Score.Zero;
		}

		score = score * this.getFatigueScoreMult(this.m.Skill);
		local opponents = this.getAgent().getKnownOpponents();
		local func = this.calculateDanger(_entity, opponents);

		while (resume func == null)
		{
			yield null;
		}

		local func = this.findBestTarget(_entity, opponents);

		while (resume func == null)
		{
			yield null;
		}

		if (this.m.TargetTile == null)
		{
			return this.Const.AI.Behavior.Score.Zero;
		}

		this.m.Danger = null;
		return this.Const.AI.Behavior.Score.Weakness * score + this.m.ScoreBonus;
	}

	function onExecute( _entity )
	{
		if (this.m.IsFirstExecuted)
		{
			this.getAgent().adjustCameraToTarget(this.m.TargetTile);
			this.m.IsFirstExecuted = false;

			if (this.m.TargetTile.IsVisibleForPlayer && _entity.isHiddenToPlayer())
			{
				_entity.setDiscovered(true);
				_entity.getTile().addVisibilityForFaction(this.Const.Faction.Player);
			}

			return false;
		}

		this.m.Skill.use(this.m.TargetTile);
		this.getAgent().declareEvaluationDelay(2000);

		if (!_entity.isHiddenToPlayer() || this.m.TargetTile.IsVisibleForPlayer)
		{
			this.getAgent().declareAction(2000);
		}

		this.m.Skill = null;
		this.m.TargetTile = null;
		return true;
	}

	function findBestTarget( _entity, _targets )
	{
		// Function is a generator.
		local myTile = _entity.getTile();
		local knownAllies = this.getAgent().getKnownAllies();
		local bestScore = 0.0;
		local bestTarget;
		local time = this.Time.getExactTime();

		foreach( opponent in _targets )
		{
			if (this.isAllottedTimeReached(time))
			{
				yield null;
				time = this.Time.getExactTime();
			}

			local target = opponent.Actor;
			local opponentTile = opponent.Actor.getTile();

			if (!this.m.Skill.isUsableOn(opponentTile))
			{
				continue;
			}

			local score = 100.0;
			local distanceToTarget = myTile.getDistanceTo(opponentTile);
			local isRangedOpponent = this.isRangedUnit(target);

			if (target.getMoraleState() == this.Const.MoraleState.Fleeing || target.getCurrentProperties().IsStunned)
			{
				continue;
			}

			if (target.getSkills().hasSkill("effects.weakness"))
			{
				continue;
			}

			score = score + target.getLevel() * 5.0;

			score = score - distanceToTarget * 3.0;
			local targets = 0;
			local targetsInRange = this.queryEnemiesInMeleeRange(1, target.getIdealRange(), target);

			foreach( t in targetsInRange )
			{
				if (t.getID() != _entity.getID() && t.getCurrentProperties().TargetAttractionMult > 1.0)
				{
					targets = ++targets;
				}
			}

			score = score + targets * 3.0;

			if (target.getCurrentProperties().IsRooted && opponentTile.getZoneOfControlCount(target.getFaction()) == 0 && !target.isArmedWithRangedWeapon())
			{
				score = score * 0.5;
			}

			if (target.isArmedWithRangedWeapon() && opponentTile.getZoneOfControlCount(target.getFaction()) != 0)
			{
				score = score * 0.5;
			}

			if (target.getType() == this.Const.EntityType.Wardog || target.getType() == this.Const.EntityType.Warhound)
			{
				score = score * 0.5;
			}

			if (isRangedOpponent)
			{
				score = score * 1.5;
			}

			if (!target.isTurnDone())
			{
				score = score * 1.25;
			}

			score = score * target.getCurrentProperties().TargetAttractionMult;

			if (score > bestScore)
			{
				bestScore = score;
				bestTarget = target;
			}
		}

		if (bestTarget != null)
		{
			this.m.TargetTile = bestTarget.getTile();
			this.m.ScoreBonus = bestScore * 0.1;
		}

		return true;
	}

	function calculateDanger( _entity, _targets )
	{
		// Function is a generator.
		local myTile = _entity.getTile();
		local currentDanger = 0.0;
		local potentialDanger = [];
		local time = this.Time.getExactTime();

		foreach( target in _targets )
		{
			if (this.isAllottedTimeReached(time))
			{
				yield null;
				time = this.Time.getExactTime();
			}

			local targetTile = target.Actor.getTile();
			local realDist = myTile.getDistanceTo(targetTile);

			if (realDist <= this.Const.AI.Behavior.RangedEngageMaxDangerDist && target.Actor.getMoraleState() != this.Const.MoraleState.Fleeing && !target.Actor.isNonCombatant() && !this.isRangedUnit(target.Actor) && target.Actor.getHitpoints() / target.Actor.getHitpointsMax() >= this.Const.AI.Behavior.RangedEngageMinDangerHitpointsPct && targetTile.getZoneOfControlCountOtherThan(target.Actor.getAlliedFactions()) < this.Const.AI.Behavior.RangedEngageIgnoreDangerMinZones)
			{
				potentialDanger.push(target.Actor.getID());
				local danger = this.getDangerFromActor(target.Actor, myTile, _entity);
				currentDanger = currentDanger + danger;
			}
		}

		this.m.Danger = {
			Danger = currentDanger,
			PotentialDanger = potentialDanger
		};
		return true;
	}

	function getDangerFromActor( _actor, _target, _entity )
	{
		local d = this.queryActorTurnsNearTarget(_actor, _target, _entity);

		if (d.Turns <= this.Const.AI.Behavior.RangedEngageKeepMinTurnsAway && d.InZonesOfControl <= 2)
		{
			if (d.InZonesOfControl != 0 || _actor.getCurrentProperties().IsStunned || _actor.getCurrentProperties().IsRooted)
			{
				return 1.0;
			}
			else
			{
				return 3.0;
			}
		}
		else
		{
			return 0.0;
		}
	}

});
