this.perk_marksman <- this.inherit("scripts/skills/skill", {
	m = {
		RangedDamageBonus = 0
	},
	function create()
	{
		this.m.ID = "perk.marksman";
		this.m.Name = "Marksman";
		this.m.Description = "Each hit by ranged weapon increase damage by 5%, stack up to 20%. Changing weapon or moving reset bonus.";
		this.m.Icon = "ui/perks/perk_marksman.png";
		this.m.Type = this.Const.SkillType.Perk | this.Const.SkillType.StatusEffect;
		this.m.Order = this.Const.SkillOrder.Perk;
		this.m.IsActive = false;
		this.m.IsStacking = false;
		this.m.IsHidden = true;
	}

	function getTooltip()
	{
		return [
			{
				id = 1,
				type = "title",
				text = this.getName()
			},
			{
				id = 2,
				type = "description",
				text = this.getDescription()
			},
			{
				id = 10,
				type = "text",
				icon = "ui/icons/ranged_skill.png",
				text = "[color=" + this.Const.UI.Color.PositiveValue + "]+" + this.m.RangedDamageBonus + "%[/color] Damage with ranged weapon"
			}
		];
	}

	function onUpdate( _properties )
	{
		if (this.m.RangedDamageBonus > 0)
		{
			local bonus = (100.0 + this.m.RangedDamageBonus) / 100
			this.logDebug("Marksman bonus = " + bonus)
			_properties.RangedDamageMult *= bonus;
		}
	}

	function onTargetHit( _skill, _targetEntity, _bodyPart, _damageInflictedHitpoints, _damageInflictedArmor )
	{
		if (_targetEntity != null && this.isKindOf(_targetEntity, "actor") && _skill.isRanged())
		{
			this.m.RangedDamageBonus = this.Math.min(20, this.m.RangedDamageBonus + 5);
			this.m.IsHidden = false;
		}
	}

	function onCombatStarted()
	{
		this.m.RangedDamageBonus = 0;
		this.m.IsHidden = true;
	}

	function onCombatFinished()
	{
		this.skill.onCombatFinished();
		this.m.RangedDamageBonus = 0;
		this.m.IsHidden = true;
	}

	function onMovementStep(_tile, _levelDifference)
	{
		this.m.RangedDamageBonus = 0;
		this.m.IsHidden = true;
	}

	function onItemEquiped(_item)
	{
		if (_item == null)
		{
			return;
		}

		if (_item.getSlotType() != this.Const.ItemSlot.Mainhand)
		{
			return;
		}

		this.m.RangedDamageBonus = 0;
		this.m.IsHidden = true;
	}
});