this.light_equipment_spec <- this.inherit("scripts/skills/skill", {
	m = {
		Stacks = 0,
		StacksNeed = 2,
		Frame = 0,
		SkillCount = 0
	},
	function create()
	{
		this.m.ID = "perk.light_equipment_spec";
		this.m.Name = "Light equipment";
		this.m.Description = "Using light equipment gives possibility to attack twice. Each level gives +2 to Maximum fatigue penalty without losing profit.";
		this.m.Icon = "ui/perks/light_equipment.png";
		this.m.Type = this.Const.SkillType.Perk | this.Const.SkillType.StatusEffect;
		this.m.Order = this.Const.SkillOrder.Perk | this.Const.SkillType.StatusEffect;
		this.m.IsActive = false;
		this.m.IsStacking = false;
		this.m.IsHidden = false;
	}

	function getDescription()
	{
		if (this.getContainer().getActor().isPlacedOnMap())
		{
			return "This character prefers light equipment to be mobile and fast. After each [color=" + this.Const.UI.Color.PositiveValue + "]" + this.m.StacksNeed + "[/color] attacks he repeats attack which cost 4AP or less for free if posible. Need [color=" + this.Const.UI.Color.PositiveValue + "]" + this.Math.max(1, this.m.StacksNeed - this.m.Stacks) + "[/color] more attack(s) to next free attack.";
		}
		
		return "This character prefers light equipment. Wearing armor and helmet with up to [color=" + this.Const.UI.Color.PositiveValue + "]" + this.getContainer().getActor().getLevel() * 2 + "[/color] total penalty to Maximum Fatigue provides double attack after each 2 attack. Heavy equipment increases amount of attack to get free attack.";
	}

	function onTargetHit( _skill, _targetEntity, _bodyPart, _damageInflictedHitpoints, _damageInflictedArmor )
	{
		if (this.Time.getFrame() != this.m.Frame && this.m.SkillCount != this.Const.SkillCounter && _skill.getActionPointCost() != 0)
		{
			++this.m.Stacks;
			this.m.Frame = this.Time.getFrame();
			this.m.SkillCount = this.Const.SkillCounter;

			if (this.m.Stacks >= this.m.StacksNeed && _skill.getActionPointCost() <= 4 && _skill.isUsable() && _targetEntity.isAlive())
			{
				local info = {
					Skill = _skill,
					TargetTile = _targetEntity.getTile()
				};

				this.Time.scheduleEvent(this.TimeUnit.Virtual, this.Const.Combat.RiposteDelay, this.onExtraAttack.bindenv(this), info);
			}
		}
	}

	function onTargetMissed( _skill, _targetEntity )
	{
		if (this.Time.getFrame() != this.m.Frame && this.m.SkillCount != this.Const.SkillCounter && _skill.getActionPointCost() != 0)
		{
			++this.m.Stacks;
			this.m.Frame = this.Time.getFrame();
			this.m.SkillCount = this.Const.SkillCounter;

			if (this.m.Stacks >= this.m.StacksNeed && _skill.getActionPointCost() <= 4 && _skill.isUsable() && _targetEntity.isAlive())
			{
				local info = {
					Skill = _skill,
					TargetTile = _targetEntity.getTile()
				};

				this.Time.scheduleEvent(this.TimeUnit.Virtual, this.Const.Combat.RiposteDelay, this.onExtraAttack.bindenv(this), info);
			}
		}
	}

	function onExtraAttack( _info )
	{
		this.Const.SkillCounter++;
		this.m.SkillCount = this.Const.SkillCounter;

		_info.Skill.useForFree(_info.TargetTile);
		this.m.Stacks = 0;
	}

	function onCombatStarted()
	{
		this.m.Stacks = 0;
		this.m.Frame = 0;
		this.m.SkillCount = 0;

		local fat = this.getContainer().getActor().getLevel() * 2;
		local body = this.getContainer().getActor().getItems().getItemAtSlot(this.Const.ItemSlot.Body);
		local head = this.getContainer().getActor().getItems().getItemAtSlot(this.Const.ItemSlot.Head);

		if (body != null)
		{
			fat = fat + body.getStaminaModifier();
		}

		if (head != null)
		{
			fat = fat + head.getStaminaModifier();
		}

		if (fat >= 0)
		{
			this.m.StacksNeed = 2;
		}
		else
		{
			this.m.StacksNeed = 2 - fat;
		}
	}

	function onCombatFinished()
	{
		this.skill.onCombatFinished();
		this.m.Stacks = 0;
		this.m.Frame = 0;
		this.m.SkillCount = 0;
	}
});
